﻿namespace Eco.Mods.TechTree
{
    using System;
	using Eco.Core.Plugins.Interfaces;
	using Eco.Core.Utils;
	using Eco.Gameplay.Components;
	using Eco.Gameplay.DynamicValues;
	using Eco.Gameplay.Items;
	using Eco.Gameplay.Players;
	using Eco.Gameplay.Skills;	
	using Eco.Gameplay.Systems.TextLinks;
	using Eco.Gameplay.Systems.Tooltip;
	using Eco.Mods.TechTree;
	using Eco.Shared.Localization;
	using Eco.Shared.Serialization;

	// Profession
	[Serialized]
    public partial class ChemSkill : Skill
    {
        public override string FriendlyName { get { return "Chemist"; } }
        public override string Description { get { return Localizer.DoStr(""); } }
		
		public static int[] SkillPointCost = { 1, 1, 1, 1, 1 };
        public override int RequiredPoint { get { return this.Level < this.MaxLevel ? SkillPointCost[this.Level] : 0; } }
        public override int PrevRequiredPoint { get { return this.Level - 1 >= 0 && this.Level - 1 < this.MaxLevel ? SkillPointCost[this.Level - 1] : 0; } }
        public override int MaxLevel { get { return 1; } }
	}
    
	public class ApplyNewRootSkills : IModKitPlugin, IInitializablePlugin
    {
        public string GetStatus()
        {
            return String.Empty;
        }

        public void Initialize(TimedTask timer)
        {
            UserManager.OnUserLoggedIn.Add(u => {
                if (!u.Skillset.HasSkill(typeof(ChemSkill)))
                    u.Skillset.LearnSkill(typeof(ChemSkill));
            });
        } 
    }
	
	[Serialized]
    [RequiresSkill(typeof(ChemSkill), 0)]
    public partial class AlchemySkill : Skill
    {
        public override string FriendlyName { get { return "Alchemy"; } }
        public override string Description { get { return Localizer.DoStr(""); } }

        public static int[] SkillPointCost = { 1, 1, 1, 1, 1 };
        public override int RequiredPoint { get { return this.Level < this.MaxLevel ? SkillPointCost[this.Level] : 0; } }
        public override int PrevRequiredPoint { get { return this.Level - 1 >= 0 && this.Level - 1 < this.MaxLevel ? SkillPointCost[this.Level - 1] : 0; } }
        public override int MaxLevel { get { return 1; } }
    }
	
	// Access via Book and Scroll
	[Serialized]
    public partial class AlchemySkillBook : SkillBook<AlchemySkill, AlchemySkillScroll>
    {
        public override string FriendlyName { get { return "Alchemy Skill Book"; } }
    }
	
	[Serialized]
    public partial class AlchemySkillScroll : SkillScroll<AlchemySkill, AlchemySkillBook>
    {
        public override string FriendlyName { get { return "Alchemy Skill Scroll"; } }
    }
	
	// Skillbuch Rezept
	public partial class AlchemySkillBookRecipe : Recipe
    {
        public AlchemySkillBookRecipe()
        {
            this.Products = new CraftingElement[]
            {
                new CraftingElement<AlchemySkillBook>(),
            };
            this.Ingredients = new CraftingElement[]
            {
                new CraftingElement<LogItem>(typeof(ResearchEfficiencySkill), 20, ResearchEfficiencySkill.MultiplicativeStrategy),
                new CraftingElement<StoneItem>(typeof(ResearchEfficiencySkill), 40, ResearchEfficiencySkill.MultiplicativeStrategy),
                new CraftingElement<IronIngotItem>(typeof(ResearchEfficiencySkill), 10, ResearchEfficiencySkill.MultiplicativeStrategy),
            };
            this.CraftMinutes = new ConstantValue(30);

            this.Initialize("Alchemy Skill Book", typeof(AlchemySkillBookRecipe));
            CraftingComponent.AddRecipe(typeof(ResearchTableObject), this);
        }
    }
	
	// Transmute of the Smith
	[Serialized]
    [RequiresSkill(typeof(AlchemySkill), 1)]
    public partial class TransmuteSmithSkill : Skill
    {
        public override string FriendlyName { get { return "Transmutation of the Smith"; } }
        public override string Description { get { return Localizer.DoStr(""); } }

        public static ModificationStrategy MultiplicativeStrategy =
            new MultiplicativeStrategy(new float[] { 1, 1 - 0.2f, 1 - 0.35f, 1 - 0.5f, 1 - 0.65f, 1 - 0.8f });
        public static ModificationStrategy AdditiveStrategy =
            new AdditiveStrategy(new float[] { 0, 0.2f, 0.35f, 0.5f, 0.65f, 0.8f });
        public static int[] SkillPointCost = { 1, 2, 2, 3, 3 };
        public override int RequiredPoint { get { return this.Level < this.MaxLevel ? SkillPointCost[this.Level] : 0; } }
        public override int PrevRequiredPoint { get { return this.Level - 1 >= 0 && this.Level - 1 < this.MaxLevel ? SkillPointCost[this.Level - 1] : 0; } }
        public override int MaxLevel { get { return 3; } }
	}
}